# Procedural Modeler for a torus
#
# This gets sourced into your tclinit block and then you can call it
# from inside the tclinit block.  Note this procedure creates geometry
# and adds it to the slide scene description.  Currently this must
# be done in the tclinit block.


proc GenerateTorus { {pcName "Torus"}     \
                     {fMajorRadius 1.0}   \
                     {fMinorRadius 0.25}  \
                     {iThetaSlices 16}    \
                     {iPhiSlices 16}      \
                     {fSMin 0.0}          \
                     {fTMin 0.0}          \
                     {fSMax 1.0}          \
                     {fTMax 1.0} } {

    # Import variable storing pi
    global SLF_PI

    # This is how much the angle changes between slices around the
    # larger radius.  Note that because we use expr we are actually 
    # evaluating the expression here.

    set fDeltaTheta [expr 2.0 * $SLF_PI / $iThetaSlices]


    # Set the initial Theta

    set fTheta 0


    # This is how much the angle changes between slices around the
    # smaller axis.   Note that because we use expr we are actually 
    # evaluating the expression here.

    set fDeltaPhi [expr -2.0 * $SLF_PI / $iPhiSlices]


    # Set the initial Phi

    set fPhi 0

    # Do the same thing for the texture coordinates
    #  Note that fS goes around the small radius and fT around the big radius.
    set fDeltaS [expr double($fSMax - $fSMin) / $iPhiSlices]
    set fS $fSMin
    set fDeltaT [expr double($fTMax - $fTMin) / $iThetaSlices]
    set fT $fTMin


    # Loop through the phi slices around the small radius
    #
    for { set j 0 } { $j <= $iPhiSlices } { incr j } {

        # Compute the normal vector in the plane containing a slice through
        # the torus
	set fRNormal [expr cos($fPhi)]
	set fZNormal [expr sin($fPhi)]
	set fR [expr $fMajorRadius + $fMinorRadius*cos($fPhi)]
	set fZ [expr $fMinorRadius*sin($fPhi)]


        # We must reset fTheta and fT each time we go around the larger radius
	set fTheta 0
	set fT $fTMin
	
	for { set i 0 } { $i <= $iThetaSlices } { incr i } {
            # This expression basically rotates the normal above into the
            # current slice around the large radius
            #
	    set vNormal [list [expr $fRNormal*cos($fTheta)] [expr $fRNormal*sin($fTheta)] $fZNormal]

            # Here we compute the actual point

	    set pt [list [expr $fR*cos($fTheta)] [expr $fR*sin($fTheta)] $fZ]

	    # this expression actually adds the point with normal and
            # texture information to the slide scene desciption.
            # the first argument is the points name.
            #
	    slide create point "p[subst $pcName]_[subst $i]_[subst $j]" $pt -normal $vNormal -texture [list $fS $fT 0]

	    # increment the theta and t values 
	    set fTheta [expr $fTheta + $fDeltaTheta]
	    set fT [expr $fT + $fDeltaT]
	}

	# increment the phi and s values 
	set fPhi [expr $fPhi + $fDeltaPhi]
	set fS [expr $fS + $fDeltaS]
    } 

    set iFaceID 0
    set listFaces ""


    # The following loop goes back through the vertices and makes
    # a slide face for each triangle of vertices on the surface.
    # At the same time it is maintaining a list of the names of the
    # faces it is making so that we can make a slide object with that
    # list (actually a long string).

    for { set i 1 } { $i <= $iThetaSlices } { incr i } {
	for { set j 1 } { $j <= $iPhiSlices } { incr j } {

            # This adds a face to the slide scene
	    slide create face "f[subst $pcName]_[subst $iFaceID]" [list "p[subst $pcName]_[expr $i - 1]_[expr $j - 1]" "p[subst $pcName]_[expr $i - 1]_[expr $j]" "p[subst $pcName]_[expr $i]_[expr $j - 1]"]

            # This adds the name of the face to the string of faces.
	    set listFaces "$listFaces f[subst $pcName]_[subst $iFaceID]"
	    incr iFaceID


            # This adds a face to the slide scene
	    slide create face "f[subst $pcName]_[subst $iFaceID]" [list "p[subst $pcName]_[expr $i]_[expr $j - 1]" "p[subst $pcName]_[expr $i - 1]_[expr $j]" "p[subst $pcName]_[expr $i]_[expr $j]"]

            # This adds the name of the face to the string of faces.
	    set listFaces "$listFaces f[subst $pcName]_[subst $iFaceID]"
	    incr iFaceID
	}
    }

    # Finally we output the object with the string of face identifiers 
    # we made earlier.  Note that we call it o+pcName
    slide create object "o[subst $pcName]" $listFaces 
}
